@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 // Source file for miscellaneous entities that are infrequently referenced by 
@@ -62,7 +54,7 @@
 namespace tbb {
 namespace internal {
 
-#if __TBB_OS_AFFINITY_SYSCALL_PRESENT
+#if __TBB_USE_OS_AFFINITY_SYSCALL
 
 static void set_affinity_mask( size_t maskSize, const basic_mask_t* threadMask ) {
 #if __linux__
@@ -249,22 +241,24 @@ struct TBB_GROUP_AFFINITY {
     WORD   Reserved[3];
 };
 
-static DWORD (WINAPI *TBB_GetMaximumProcessorCount)( WORD groupIndex ) = NULL;
-static WORD (WINAPI *TBB_GetMaximumProcessorGroupCount)() = NULL;
+static DWORD (WINAPI *TBB_GetActiveProcessorCount)( WORD groupIndex ) = NULL;
+static WORD (WINAPI *TBB_GetActiveProcessorGroupCount)() = NULL;
 static BOOL (WINAPI *TBB_SetThreadGroupAffinity)( HANDLE hThread, 
                         const TBB_GROUP_AFFINITY* newAff, TBB_GROUP_AFFINITY *prevAff );
 static BOOL (WINAPI *TBB_GetThreadGroupAffinity)( HANDLE hThread, TBB_GROUP_AFFINITY* );
 
 static const dynamic_link_descriptor ProcessorGroupsApiLinkTable[] = {
-      DLD(GetMaximumProcessorCount, TBB_GetMaximumProcessorCount)
-    , DLD(GetMaximumProcessorGroupCount, TBB_GetMaximumProcessorGroupCount)
+      DLD(GetActiveProcessorCount, TBB_GetActiveProcessorCount)
+    , DLD(GetActiveProcessorGroupCount, TBB_GetActiveProcessorGroupCount)
     , DLD(SetThreadGroupAffinity, TBB_SetThreadGroupAffinity)
     , DLD(GetThreadGroupAffinity, TBB_GetThreadGroupAffinity)
 };
 
 static void initialize_hardware_concurrency_info () {
 #if __TBB_WIN8UI_SUPPORT
-    unsigned int nproc = std::thread::hardware_concurrency();
+    // For these applications processor groups info is unavailable
+    // Setting up a number of processors for one processor group
+    theProcessorGroups[0].numProcs = theProcessorGroups[0].numProcsRunningTotal = std::thread::hardware_concurrency();
 #else /* __TBB_WIN8UI_SUPPORT */
     dynamic_link( "Kernel32.dll", ProcessorGroupsApiLinkTable,
                   sizeof(ProcessorGroupsApiLinkTable)/sizeof(dynamic_link_descriptor) );
@@ -278,9 +272,12 @@ static void initialize_hardware_concurrency_info () {
             ++nproc;
     }
     __TBB_ASSERT( nproc <= (int)si.dwNumberOfProcessors, NULL );
-    if ( nproc == (int)si.dwNumberOfProcessors && TBB_GetMaximumProcessorCount ) {
+    // By default setting up a number of processors for one processor group
+    theProcessorGroups[0].numProcs = theProcessorGroups[0].numProcsRunningTotal = nproc;
+    // Setting up processor groups in case the process does not restrict affinity mask and more than one processor group is present
+    if ( nproc == (int)si.dwNumberOfProcessors && TBB_GetActiveProcessorCount ) {
         // The process does not have restricting affinity mask and multiple processor groups are possible
-        ProcessorGroupInfo::NumGroups = (int)TBB_GetMaximumProcessorGroupCount();
+        ProcessorGroupInfo::NumGroups = (int)TBB_GetActiveProcessorGroupCount();
         __TBB_ASSERT( ProcessorGroupInfo::NumGroups <= MaxProcessorGroups, NULL );
         // Fail safety bootstrap. Release versions will limit available concurrency
         // level, while debug ones would assert.
@@ -293,18 +290,15 @@ static void initialize_hardware_concurrency_info () {
             int nprocs = 0;
             for ( WORD i = 0; i < ProcessorGroupInfo::NumGroups; ++i ) {
                 ProcessorGroupInfo  &pgi = theProcessorGroups[i];
-                pgi.numProcs = (int)TBB_GetMaximumProcessorCount(i);
+                pgi.numProcs = (int)TBB_GetActiveProcessorCount(i);
                 __TBB_ASSERT( pgi.numProcs <= (int)sizeof(DWORD_PTR) * CHAR_BIT, NULL );
                 pgi.mask = pgi.numProcs == sizeof(DWORD_PTR) * CHAR_BIT ? ~(DWORD_PTR)0 : (DWORD_PTR(1) << pgi.numProcs) - 1;
                 pgi.numProcsRunningTotal = nprocs += pgi.numProcs;
             }
-            __TBB_ASSERT( nprocs == (int)TBB_GetMaximumProcessorCount( TBB_ALL_PROCESSOR_GROUPS ), NULL );
-            return;
+            __TBB_ASSERT( nprocs == (int)TBB_GetActiveProcessorCount( TBB_ALL_PROCESSOR_GROUPS ), NULL );
         }
     }
 #endif /* __TBB_WIN8UI_SUPPORT */
-    // Either the process has restricting affinity mask or only a single processor groups is present
-    theProcessorGroups[0].numProcs = theProcessorGroups[0].numProcsRunningTotal = nproc;
 
     PrintExtraVersionInfo("Processor groups", "%d", ProcessorGroupInfo::NumGroups);
     if (ProcessorGroupInfo::NumGroups>1)
