@@ -60,4 +60,21 @@ inline size_t arrayLength(const T(&)[N]) {
     return N;
 }
 
+namespace rml {
+namespace internal {
+
+/*
+ * Best estimate of cache line size, for the purpose of avoiding false sharing.
+ * Too high causes memory overhead, too low causes false-sharing overhead.
+ * Because, e.g., 32-bit code might run on a 64-bit system with a larger cache line size,
+ * it would probably be better to probe at runtime where possible and/or allow for an environment variable override,
+ * but currently this is still used for compile-time layout of class Block, so the change is not entirely trivial.
+ */
+#if __powerpc64__ || __ppc64__ || __bgp__
+const uint32_t estimatedCacheLineSize = 128;
+#else
+const uint32_t estimatedCacheLineSize =  64;
+#endif
+
+}} // namespaces
 #endif /* __TBB_shared_utils_H */
