@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "tbb/scalable_allocator.h"
@@ -316,29 +308,60 @@ static void *fixedBufGetMem(intptr_t /*pool_id*/, size_t &bytes)
     return buf;
 }
 
+class FixedPoolRun: NoAssign {
+    Harness::SpinBarrier *startB;
+    rml::MemoryPool *pool;
+    size_t reqSize;
+public:
+    FixedPoolRun(Harness::SpinBarrier *b, rml::MemoryPool *p, size_t sz) :
+        startB(b), pool(p), reqSize(sz) {}
+    void operator()( int /*id*/ ) const {
+        const int ITERS = 10000;
+        startB->wait();
+        for (int i=0; i<ITERS; i++) {
+            void *o = pool_malloc(pool, reqSize);
+            ASSERT(o, NULL);
+            pool_free(pool, o);
+        }
+    }
+};
+
 void TestFixedBufferPool()
 {
-    void *ptrs[7];
+    const int ITERS = 7;
+    const size_t MAX_OBJECT = 7*1024*1024;
+    void *ptrs[ITERS];
     rml::MemPoolPolicy pol(fixedBufGetMem, NULL, 0, /*fixedSizePool=*/true,
                            /*keepMemTillDestroy=*/false);
     rml::MemoryPool *pool;
 
     pool_create_v1(0, &pol, &pool);
-    void *largeObj = pool_malloc(pool, 7*1024*1024);
+    void *largeObj = pool_malloc(pool, MAX_OBJECT);
     ASSERT(largeObj, NULL);
     pool_free(pool, largeObj);
 
-    for (int i=0; i<7; i++) {
-        ptrs[i] = pool_malloc(pool, 1024*1024);
+    largeObj = pool_malloc(pool, MAX_OBJECT);
+    ASSERT(largeObj, NULL);
+    pool_free(pool, largeObj);
+
+    for (int i=0; i<ITERS; i++) {
+        ptrs[i] = pool_malloc(pool, MAX_OBJECT/ITERS);
         ASSERT(ptrs[i], NULL);
     }
-    for (int i=0; i<7; i++)
+    for (int i=0; i<ITERS; i++)
         pool_free(pool, ptrs[i]);
 
-    largeObj = pool_malloc(pool, 7*1024*1024);
+    largeObj = pool_malloc(pool, MAX_OBJECT);
     ASSERT(largeObj, NULL);
     pool_free(pool, largeObj);
 
+    // each thread asks for an MAX_OBJECT/p/2 object,
+    // /2 is to cover fragmentation
+    for (int p=MinThread; p<=MaxThread; p++) {
+        Harness::SpinBarrier startB(p);
+        NativeParallelFor( p, FixedPoolRun(&startB, pool,
+                                           MAX_OBJECT/p/2) );
+    }
     pool_destroy(pool);
 }
 
@@ -357,7 +380,7 @@ static int putGranMem(intptr_t /*pool_id*/, void *ptr, size_t bytes)
     return 0;
 }
 
-static void TestPoolGranularity()
+void TestPoolGranularity()
 {
     rml::MemPoolPolicy pol(getGranMem, putGranMem);
     const size_t grans[] = {4*1024, 2*1024*1024, 6*1024*1024, 10*1024*1024};
@@ -391,7 +414,7 @@ static int putMemPolicy(intptr_t /*pool_id*/, void *ptr, size_t /*bytes*/)
     return 0;
 }
 
-static void TestPoolKeepTillDestroy()
+void TestPoolKeepTillDestroy()
 {
     const int ITERS = 50*1024;
     void *ptrs[2*ITERS+1];
@@ -445,7 +468,7 @@ static bool memEqual(char *buf, size_t size, int val)
     return memEq;
 }
 
-static void TestEntries()
+void TestEntries()
 {
     const int SZ = 4;
     const int ALGN = 4;
@@ -481,7 +504,7 @@ static void TestEntries()
     pool_destroy(pool);
 }
 
-static void TestPoolCreation()
+void TestPoolCreation()
 {
     using namespace rml;
 
