@@ -1,29 +1,21 @@
 /*
-    Copyright 2005-2012 Intel Corporation.  All Rights Reserved.
+    Copyright 2005-2015 Intel Corporation.  All Rights Reserved.
 
-    This file is part of Threading Building Blocks.
+    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
+    you can redistribute it and/or modify it under the terms of the GNU General Public License
+    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
+    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
+    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+    See  the GNU General Public License for more details.   You should have received a copy of
+    the  GNU General Public License along with Threading Building Blocks; if not, write to the
+    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
 
-    Threading Building Blocks is free software; you can redistribute it
-    and/or modify it under the terms of the GNU General Public License
-    version 2 as published by the Free Software Foundation.
-
-    Threading Building Blocks is distributed in the hope that it will be
-    useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
-
-    You should have received a copy of the GNU General Public License
-    along with Threading Building Blocks; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
-
-    As a special exception, you may use this file as part of a free software
-    library without restriction.  Specifically, if other files instantiate
-    templates or use macros or inline functions from this file, or you compile
-    this file and link it with other files to produce an executable, this
-    file does not by itself cause the resulting executable to be covered by
-    the GNU General Public License.  This exception does not however
-    invalidate any other reasons why the executable file might be covered by
-    the GNU General Public License.
+    As a special exception,  you may use this file  as part of a free software library without
+    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
+    functions from this file, or you compile this file and link it with other files to produce
+    an executable,  this file does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however invalidate any other
+    reasons why the executable file might be covered by the GNU General Public License.
 */
 
 #include "harness_task.h"
@@ -285,7 +277,7 @@ static int TestUnconstructibleTaskCount;
 struct ConstructionFailure {
 };
 
-#if _MSC_VER && !defined(__INTEL_COMPILER)
+#if __TBB_MSVC_UNREACHABLE_CODE_IGNORED
     // Suppress pointless "unreachable code" warning.
     #pragma warning (push)
     #pragma warning (disable: 4702)
@@ -300,7 +292,7 @@ struct UnconstructibleTask: public tbb::empty_task {
     }
 };
 
-#if _MSC_VER && !defined(__INTEL_COMPILER)
+#if __TBB_MSVC_UNREACHABLE_CODE_IGNORED
     #pragma warning (pop)
 #endif
 
@@ -475,7 +467,7 @@ public:
         }
         if( DagTask* t = successor_to_below ) {
             t->sum_from_above = sum;
-            if( t->decrement_ref_count()==0 )
+            if( t->add_ref_count(-1)==0 )
                 // Test using bypass to evaluate DAG
                 return t;
         }
@@ -783,6 +775,26 @@ void TestMastersIsolation ( int p ) {
     }
 }
 
+struct waitable_task : tbb::task {
+    tbb::task* execute() {
+        recycle_as_safe_continuation(); // do not destroy the task after execution
+        set_parent(this);               // decrement its own ref_count after completion
+        __TBB_Yield();
+        return NULL;
+    }
+};
+void TestWaitableTask() {
+    waitable_task &wt = *new( tbb::task::allocate_root() ) waitable_task;
+    for( int i = 0; i < 100000; i++ ) {
+        wt.set_ref_count(2);            // prepare for waiting on it
+        wt.spawn(wt);
+        if( i&1 ) __TBB_Yield();
+        wt.wait_for_all();
+    }
+    wt.set_parent(NULL);                // prevents assertions and atomics in task::destroy
+    tbb::task::destroy(wt);
+}
+
 int TestMain () {
 #if TBB_USE_EXCEPTIONS
     TestUnconstructibleTask<1>();
@@ -805,5 +817,6 @@ int TestMain () {
         TestRelaxedOwnership( p );
         TestMastersIsolation( p );
     }
+    TestWaitableTask();
     return Harness::Done;
 }
